Package: torchvision
Title: Models, Datasets and Transformations for Images
Version: 0.9.0
Authors@R: c(
    person("Tomasz", "Kalinowski", role = c("ctb", "cre"),
           email = "tomasz@posit.co"),
    person("Daniel", "Falbel", role = c("aut", "cph"),
           email = "dfalbel@gmail.com"),
    person("Christophe", "Regouby", role = c("ctb"),
           email = "christophe.regouby@free.fr"),
    person("Akanksha", "Koshti", role = c("ctb")),
    person("Derrick", "Richard", role = c("ctb"),
           email = "derrickrds@gmail.com"),
    person("ANAMASGARD", role = c("ctb")),
    person("Chandraveer", "Singh", role = c("ctb"),
           email = "chandraveersingh1717@gmail.com"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
    )
Description: Provides access to datasets, models and preprocessing
    facilities for deep learning with images. Integrates seamlessly
    with the 'torch' package and its API borrows heavily from
    the 'PyTorch' vision package.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://torchvision.mlverse.org,
        https://github.com/mlverse/torchvision
RoxygenNote: 7.3.3
Imports: torch (>= 0.5.0), fs, rlang, rappdirs, utils, jpeg, tiff,
        magrittr, png, abind, jsonlite, withr, cli, glue, zeallot
Suggests: arrow, magick, prettyunits, testthat, coro, R.matlab, xml2,
        knitr, rmarkdown, torchvisionlib
BugReports: https://github.com/mlverse/torchvision/issues
Collate: 'collection-catalog.R' 'folder-dataset.R'
        'collection-rf100-doc.R' 'collection-rf100-biology.R'
        'collection-rf100-damage.R' 'collection-rf100-infrared.R'
        'collection-rf100-medical.R' 'collection-rf100-underwater.R'
        'conditions.R' 'dataset-caltech.R' 'dataset-cifar.R'
        'dataset-coco.R' 'dataset-eurosat.R' 'dataset-fer.R'
        'dataset-fgvc.R' 'dataset-flickr.R' 'dataset-flowers.R'
        'dataset-imagenet.R' 'dataset-lfw.R' 'dataset-mnist.R'
        'dataset-oxfordiiitpet.R' 'dataset-pascal.R'
        'dataset-places365.R' 'dataset-plankton.R'
        'dataset-rf100-peixos.R' 'dataset-vggface2.R' 'extension.R'
        'globals.R' 'models-alexnet.R' 'models-convnext.R'
        'models-convnext_detection.R' 'models-convnext_segmentation.R'
        'models-deeplabv3.R' 'models-efficientnet.R'
        'models-efficientnetv2.R' 'models-facenet.R'
        'models-faster_rcnn.R' 'models-fcn.R' 'models-inception.R'
        'models-mask_rcnn.R' 'models-maxvit.R' 'models-mobilenetv2.R'
        'models-mobilenetv3.R' 'models-mobilenetv3_large.R'
        'models-resnet.R' 'models-vgg.R' 'models-vit.R'
        'ops-box_convert.R' 'ops-boxes.R' 'transforms-array.R'
        'transforms-defaults.R' 'transforms-generics.R'
        'transforms-magick.R' 'transforms-segmentation.R'
        'transforms-tensor.R' 'utils.R' 'vision_utils.R'
Depends: R (>= 3.5)
LazyData: true
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-22 13:10:42 UTC; dfalbel
Author: Tomasz Kalinowski [ctb, cre],
  Daniel Falbel [aut, cph],
  Christophe Regouby [ctb],
  Akanksha Koshti [ctb],
  Derrick Richard [ctb],
  ANAMASGARD [ctb],
  Chandraveer Singh [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-04-22 14:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 03:05:01 UTC; unix
