# Generated by roxygen2: do not edit by hand

S3method(draw_bounding_boxes,default)
S3method(draw_bounding_boxes,image_with_bounding_box)
S3method(draw_bounding_boxes,torch_tensor)
S3method(draw_segmentation_masks,default)
S3method(draw_segmentation_masks,image_with_segmentation_mask)
S3method(draw_segmentation_masks,torch_tensor)
S3method(get_image_size,"magick-image")
S3method(transform_adjust_brightness,default)
S3method(transform_adjust_brightness,torch_tensor)
S3method(transform_adjust_contrast,default)
S3method(transform_adjust_contrast,torch_tensor)
S3method(transform_adjust_gamma,default)
S3method(transform_adjust_gamma,torch_tensor)
S3method(transform_adjust_hue,default)
S3method(transform_adjust_hue,torch_tensor)
S3method(transform_adjust_saturation,default)
S3method(transform_adjust_saturation,torch_tensor)
S3method(transform_affine,default)
S3method(transform_affine,torch_tensor)
S3method(transform_center_crop,default)
S3method(transform_color_jitter,default)
S3method(transform_convert_image_dtype,default)
S3method(transform_convert_image_dtype,torch_tensor)
S3method(transform_crop,"magick-image")
S3method(transform_crop,default)
S3method(transform_crop,torch_tensor)
S3method(transform_five_crop,default)
S3method(transform_five_crop,torch_tensor)
S3method(transform_grayscale,default)
S3method(transform_grayscale,torch_tensor)
S3method(transform_hflip,"magick-image")
S3method(transform_hflip,default)
S3method(transform_hflip,torch_tensor)
S3method(transform_linear_transformation,default)
S3method(transform_linear_transformation,torch_tensor)
S3method(transform_normalize,default)
S3method(transform_normalize,torch_tensor)
S3method(transform_pad,default)
S3method(transform_pad,torch_tensor)
S3method(transform_perspective,default)
S3method(transform_perspective,torch_tensor)
S3method(transform_random_affine,default)
S3method(transform_random_apply,default)
S3method(transform_random_choice,default)
S3method(transform_random_crop,default)
S3method(transform_random_erasing,default)
S3method(transform_random_grayscale,default)
S3method(transform_random_grayscale,torch_tensor)
S3method(transform_random_horizontal_flip,default)
S3method(transform_random_order,default)
S3method(transform_random_perspective,default)
S3method(transform_random_resized_crop,default)
S3method(transform_random_rotation,default)
S3method(transform_random_vertical_flip,default)
S3method(transform_resize,"magick-image")
S3method(transform_resize,default)
S3method(transform_resize,torch_tensor)
S3method(transform_resized_crop,default)
S3method(transform_rgb_to_grayscale,torch_tensor)
S3method(transform_rotate,default)
S3method(transform_rotate,torch_tensor)
S3method(transform_ten_crop,default)
S3method(transform_ten_crop,torch_tensor)
S3method(transform_to_tensor,"magick-image")
S3method(transform_to_tensor,array)
S3method(transform_to_tensor,default)
S3method(transform_to_tensor,list)
S3method(transform_to_tensor,matrix)
S3method(transform_vflip,default)
S3method(transform_vflip,torch_tensor)
export(base_loader)
export(batched_nms)
export(box_area)
export(box_convert)
export(box_iou)
export(caltech101_dataset)
export(caltech256_dataset)
export(caltech_classes)
export(cifar100_dataset)
export(cifar10_dataset)
export(clip_boxes_to_image)
export(coco_caption_dataset)
export(coco_classes)
export(coco_detection_dataset)
export(coco_segmentation_dataset)
export(draw_bounding_boxes)
export(draw_keypoints)
export(draw_segmentation_masks)
export(emnist_collection)
export(emnist_dataset)
export(eurosat100_dataset)
export(eurosat_all_bands_dataset)
export(eurosat_dataset)
export(fashion_mnist_dataset)
export(fer_dataset)
export(fgvc_aircraft_dataset)
export(flickr30k_caption_dataset)
export(flickr8k_caption_dataset)
export(flowers102_dataset)
export(generalized_box_iou)
export(get_collection_catalog)
export(image_folder_dataset)
export(imagenet_1k_classes)
export(imagenet_21k_classes)
export(imagenet_21k_df)
export(imagenet_classes)
export(kmnist_dataset)
export(lfw_pairs_dataset)
export(lfw_people_dataset)
export(list_collection_datasets)
export(magick_loader)
export(mnist_dataset)
export(model_alexnet)
export(model_convnext_base_1k)
export(model_convnext_base_22k)
export(model_convnext_base_detection)
export(model_convnext_base_fcn)
export(model_convnext_base_upernet)
export(model_convnext_large_1k)
export(model_convnext_large_22k)
export(model_convnext_small_22k)
export(model_convnext_small_22k1k)
export(model_convnext_small_detection)
export(model_convnext_small_fcn)
export(model_convnext_small_upernet)
export(model_convnext_tiny_1k)
export(model_convnext_tiny_22k)
export(model_convnext_tiny_detection)
export(model_convnext_tiny_fcn)
export(model_convnext_tiny_upernet)
export(model_deeplabv3_resnet101)
export(model_deeplabv3_resnet50)
export(model_efficientnet_b0)
export(model_efficientnet_b1)
export(model_efficientnet_b2)
export(model_efficientnet_b3)
export(model_efficientnet_b4)
export(model_efficientnet_b5)
export(model_efficientnet_b6)
export(model_efficientnet_b7)
export(model_efficientnet_v2_l)
export(model_efficientnet_v2_m)
export(model_efficientnet_v2_s)
export(model_facenet_inception_resnet_v1)
export(model_facenet_onet)
export(model_facenet_pnet)
export(model_facenet_rnet)
export(model_fasterrcnn_mobilenet_v3_large_320_fpn)
export(model_fasterrcnn_mobilenet_v3_large_fpn)
export(model_fasterrcnn_resnet50_fpn)
export(model_fasterrcnn_resnet50_fpn_v2)
export(model_fcn_resnet101)
export(model_fcn_resnet50)
export(model_inception_v3)
export(model_maskrcnn_resnet50_fpn)
export(model_maskrcnn_resnet50_fpn_v2)
export(model_maxvit)
export(model_mobilenet_v2)
export(model_mobilenet_v3_large)
export(model_mobilenet_v3_large_quantized)
export(model_mobilenet_v3_small)
export(model_mtcnn)
export(model_resnet101)
export(model_resnet152)
export(model_resnet18)
export(model_resnet34)
export(model_resnet50)
export(model_resnext101_32x8d)
export(model_resnext50_32x4d)
export(model_vgg11)
export(model_vgg11_bn)
export(model_vgg13)
export(model_vgg13_bn)
export(model_vgg16)
export(model_vgg16_bn)
export(model_vgg19)
export(model_vgg19_bn)
export(model_vit_b_16)
export(model_vit_b_32)
export(model_vit_h_14)
export(model_vit_l_16)
export(model_vit_l_32)
export(model_wide_resnet101_2)
export(model_wide_resnet50_2)
export(nms)
export(oxfordiiitpet_binary_dataset)
export(oxfordiiitpet_dataset)
export(oxfordiiitpet_segmentation_dataset)
export(pascal_detection_dataset)
export(pascal_segmentation_dataset)
export(pascal_voc_classes)
export(places365_dataset)
export(places365_dataset_large)
export(qmnist_dataset)
export(remove_small_boxes)
export(rf100_biology_collection)
export(rf100_damage_collection)
export(rf100_document_collection)
export(rf100_infrared_collection)
export(rf100_medical_collection)
export(rf100_peixos_segmentation_dataset)
export(rf100_underwater_collection)
export(search_collection)
export(target_transform_coco_masks)
export(target_transform_trimap_masks)
export(tensor_image_browse)
export(tensor_image_display)
export(tiny_imagenet_dataset)
export(transform_adjust_brightness)
export(transform_adjust_contrast)
export(transform_adjust_gamma)
export(transform_adjust_hue)
export(transform_adjust_saturation)
export(transform_affine)
export(transform_center_crop)
export(transform_color_jitter)
export(transform_convert_image_dtype)
export(transform_crop)
export(transform_five_crop)
export(transform_grayscale)
export(transform_hflip)
export(transform_linear_transformation)
export(transform_normalize)
export(transform_pad)
export(transform_perspective)
export(transform_random_affine)
export(transform_random_apply)
export(transform_random_choice)
export(transform_random_crop)
export(transform_random_erasing)
export(transform_random_grayscale)
export(transform_random_horizontal_flip)
export(transform_random_order)
export(transform_random_perspective)
export(transform_random_resized_crop)
export(transform_random_rotation)
export(transform_random_vertical_flip)
export(transform_resize)
export(transform_resized_crop)
export(transform_rgb_to_grayscale)
export(transform_rotate)
export(transform_ten_crop)
export(transform_to_tensor)
export(transform_vflip)
export(vggface2_dataset)
export(vision_make_grid)
export(whoi_plankton_dataset)
export(whoi_small_coralnet_dataset)
export(whoi_small_plankton_dataset)
importFrom(grDevices,dev.off)
importFrom(graphics,polygon)
importFrom(jsonlite,fromJSON)
importFrom(magrittr,"%>%")
importFrom(stats,setNames)
importFrom(torch,as_array)
importFrom(torch,dataset)
importFrom(torch,load_state_dict)
importFrom(torch,nn_adaptive_avg_pool2d)
importFrom(torch,nn_batch_norm1d)
importFrom(torch,nn_batch_norm2d)
importFrom(torch,nn_conv2d)
importFrom(torch,nn_conv_transpose2d)
importFrom(torch,nn_dropout)
importFrom(torch,nn_gelu)
importFrom(torch,nn_hardsigmoid)
importFrom(torch,nn_hardswish)
importFrom(torch,nn_identity)
importFrom(torch,nn_layer_norm)
importFrom(torch,nn_linear)
importFrom(torch,nn_max_pool2d)
importFrom(torch,nn_module)
importFrom(torch,nn_module_dict)
importFrom(torch,nn_module_list)
importFrom(torch,nn_parameter)
importFrom(torch,nn_prelu)
importFrom(torch,nn_relu)
importFrom(torch,nn_sequential)
importFrom(torch,nn_softmax)
importFrom(torch,nnf_gelu)
importFrom(torch,nnf_grid_sample)
importFrom(torch,nnf_interpolate)
importFrom(torch,nnf_layer_norm)
importFrom(torch,nnf_normalize)
importFrom(torch,nnf_relu)
importFrom(torch,nnf_softmax)
importFrom(torch,torch_arange)
importFrom(torch,torch_cat)
importFrom(torch,torch_chunk)
importFrom(torch,torch_clamp)
importFrom(torch,torch_empty)
importFrom(torch,torch_flatten)
importFrom(torch,torch_float)
importFrom(torch,torch_float32)
importFrom(torch,torch_int32)
importFrom(torch,torch_linspace)
importFrom(torch,torch_log2)
importFrom(torch,torch_long)
importFrom(torch,torch_matmul)
importFrom(torch,torch_max)
importFrom(torch,torch_meshgrid)
importFrom(torch,torch_nonzero)
importFrom(torch,torch_ones)
importFrom(torch,torch_randn)
importFrom(torch,torch_sigmoid)
importFrom(torch,torch_stack)
importFrom(torch,torch_tensor)
importFrom(torch,torch_zeros)
importFrom(torch,torch_zeros_like)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,tail)
importFrom(zeallot,"%<-%")
