
# torchvision <img src='https://torch.mlverse.org/css/images/hex/torchvision.png' align="right" style="width: 15%"/>

<!-- badges: start -->
[![R build status](https://github.com/mlverse/torchvision/workflows/R-CMD-check/badge.svg)](https://github.com/mlverse/torchvision/actions)
[![CRAN status](https://www.r-pkg.org/badges/version/torchvision)](https://CRAN.R-project.org/package=torchvision)
[![](https://cranlogs.r-pkg.org/badges/torchvision)](https://cran.r-project.org/package=torchvision)
<!-- badges: end -->

torchvision is an extension for [torch](https://github.com/mlverse/torch) providing image loading, transformations, common architectures for computer vision, pre-trained weights and access to commonly used datasets. 

## Installation

The CRAN release can be installed with:

```r
install.packages("torchvision")
```

You can install the development version from GitHub with:

``` r
remotes::install_github("mlverse/torchvision@main")
```

