

set -ex



pip check
python test/smoke_test.py
pytest --verbose --durations=50 -k "not (test_internet or test_detection_model or test_quantized_classification_model or test_forward_backward or test_jit_forward_backward)" test/test_image.py
pytest --verbose --durations=50 -k "not (test_internet or test_detection_model or test_quantized_classification_model or test_forward_backward or test_jit_forward_backward)" test/test_transforms_v2.py
pytest --verbose --durations=50 -k "not (test_internet or test_detection_model or test_quantized_classification_model or test_forward_backward or test_jit_forward_backward)" test/test_datasets.py
pytest --verbose --durations=50 -k "not (test_internet or test_detection_model or test_quantized_classification_model or test_forward_backward or test_jit_forward_backward)" test/test_ops.py
exit 0
