% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collection-catalog.R
\name{get_collection_catalog}
\alias{get_collection_catalog}
\title{Get Complete Collection Catalog}
\usage{
get_collection_catalog()
}
\value{
A data frame with all datasets and their metadata.
}
\description{
Returns the complete catalog of datasets in collections with their metadata.
This is a convenience function that loads and returns the collection_catalog data.
}
\examples{
\dontrun{
# Get complete catalog
catalog <- get_collection_catalog()

# View in RStudio
View(catalog)

# Summary statistics
summary(catalog$total_size_mb)
table(catalog$collection)

# Find smallest dataset
catalog[which.min(catalog$total_size_mb), ]

# Find largest dataset
catalog[which.max(catalog$total_size_mb), ]
}

}
\seealso{
\code{\link[=search_collection]{search_collection()}}, \link{collection_catalog}
}
