% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collection-catalog.R
\name{list_collection_datasets}
\alias{list_collection_datasets}
\title{List Datasets in a Collection}
\usage{
list_collection_datasets(collection)
}
\arguments{
\item{collection}{Collection name. One of: "biology", "medical", "infrared",
"damage", "underwater", "document".}
}
\value{
Character vector of dataset names in the collection.
}
\description{
List all available datasets within a specific RF100 collection.
}
\examples{
\dontrun{
# List all biology datasets
list_collection_datasets("biology")

# List all medical datasets
list_collection_datasets("medical")
}

}
\seealso{
\code{\link[=search_collection]{search_collection()}}, \code{\link[=get_collection_catalog]{get_collection_catalog()}}
}
