% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/folder-dataset.R
\name{base_loader}
\alias{base_loader}
\title{Base loader}
\usage{
base_loader(path)
}
\arguments{
\item{path}{path or URL to load the image from.}
}
\value{
an channel-last array of image values with dim Height x Width x 3
}
\description{
Loads an image using \code{jpeg}, \code{png} or \code{tiff} packages depending on the
file extension.
}
